# severity = brutal

verbose = 9

[-ValuesAndExpressions::ProhibitAccessOfPrivateData]
[-Subroutines::ProhibitSubroutinePrototypes]
[-Subroutines::ProhibitExplicitReturnUndef]

[TestingAndDebugging::RequireUseStrict]
equivalent_modules = Moo Moo::Role strictures Test2::V0
